let components = [];

/*
  QuickUnion is faster than QuickFind,
  but also too slow.
  QuickUnion's defect is that it's trees
  could get too tall.
  This would mean that the find operation
  would be too expensive.
  You could end up with 1 long skinny tree,
  with one object pointing to the next.
  Doing a find operation for a node on the bottom,
  would involve going all the way to the tree
  involving N array access just for find (too slow)
*/
class QuickUnion {

  constructor(args) {
    for(let i=0;i<args.length;i++) {
      components.push(args[i]);
    }
  }

  root(i) {
    while(i !== components[i]) {
      i = components[i];
    }
    return i;
  }

  /* */
  connected(p,q) {
    root(p) === root(q);
  }

  /* Union is fast in QuickUnion
     since you just the roots of components.
     So you just end up changing 1 value in an array,
     to merge 2 components.
  */
  union(p,q) {
    let i = root(p);
    let j = root(q);
    components[i] = j;
  }
}